#include "ipaddr.h"

#ifdef WIN32

#include <winsock2.h>
#include <stdio.h>

/* Retrieve IP address via winsock2    */
/* TODO:  Retrieve MAC address as well */
int GetIP(const char *ignore, char *ip, Uint8 mac[6]) 
{
  int error;
  char hostNameBuf[1000]; // should do...
  WORD requestedVersion = 2;
  WSADATA wsaData;
  int retval=0;
  struct hostent* hostInfo;

  if (error=WSAStartup(requestedVersion, &wsaData))
  {
	fprintf(stderr,"Unable to initialize winsock\n");
	fprintf(stderr,"Error code: %s\n",WSAGetLastError());
    return(-1);
  }

  gethostname(hostNameBuf,sizeof(hostNameBuf));
  hostInfo = gethostbyname(hostNameBuf);

  if (hostInfo)
  {
    struct sockaddr_in ipAddr;
    unsigned long lAddr;
    u_long addr  = inet_addr(hostInfo->h_addr);
    char* addrPtr = (char *)(&lAddr);

	memcpy(addrPtr,(char *)(hostInfo->h_addr),hostInfo->h_length);
    ipAddr.sin_addr.s_addr = lAddr;	
    
	sprintf(ip,"%s",inet_ntoa(ipAddr.sin_addr));
  }
  else
    retval=-1;

  WSACleanup();

  return retval;
}


#else	/* Linux	*/
#	include <stdio.h>
#	include <unistd.h>
#	include <sys/socket.h>
#	include <sys/ioctl.h>
#	include <net/if.h>
#	include <net/ethernet.h>
#	include <arpa/inet.h>
#	include <string.h>

/* retrieve IP address via ioctl interface */
int GetIP(const char *Interface, char *ip,Uint8 mac[6])
{
  int                  s;
  struct ifreq         ifr;
  int                  err;
  struct sockaddr_in * addr_in;

  s = socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );
  if ( -1 == s )
  {
    perror("Getting interface socket");
    close(s);
    return(-1);
  }

  strncpy( ifr.ifr_name, Interface, IFNAMSIZ );
  err = ioctl( s, SIOCGIFADDR, &ifr );
  if ( -1 == err )
  {
    perror("Getting IP address");
    close(s);
    return(-1);
  }
  addr_in = (struct sockaddr_in *)&ifr.ifr_addr;

  if(ip != NULL)
  {
    sprintf(ip,"%s",inet_ntoa(addr_in->sin_addr));
  }

  err = ioctl(s, SIOCGIFHWADDR, &ifr );

  strncpy( ifr.ifr_name, Interface, IFNAMSIZ );
  if ( -1 == err )
  {
    perror("Getting HW address");
    close(s);
    return(-1);
  }

  bcopy( ifr.ifr_hwaddr.sa_data, mac, ETH_ALEN );

  close(s);
  return(0);
}

#endif